﻿
DeclareModule WS
  ServerEnable=#False
  ServerPort.u = 80
  IPv6=#False
  
  Structure SensorInfo
    Name.s ; Имя датчика.
    Temp.s ; Температура.
  EndStructure
  
  ;NewList Sensors.SensorInfo()
  
  Declare StartServer(Port.u, IPv6=#False)
  Declare StopServer()
  Declare ServerEvent()
  Declare UpdateInfo(List Sensors.SensorInfo())
EndDeclareModule

Module WS
  EnableExplicit
  
  Global g_ServerID=0, *g_HTML_Data=0
  
  #ServerID = 2
  
  InitNetwork()
  
  Procedure Send_Data(ClientID, Status.s, *Data, Size, Content_Type.s="text/html") ; Отправка ответа браузеру
    If *Data=0
      Size=0
    EndIf
    
    SendNetworkString(ClientID, "HTTP/1.1 "+Status+#CRLF$)
    SendNetworkString(ClientID, "Content-Length: "+Size+#CRLF$)
    SendNetworkString(ClientID, "Content-Type: "+Content_Type+#CRLF$)
    SendNetworkString(ClientID, #CRLF$)
    If *Data
      SendNetworkData(ClientID, *Data, Size)
    EndIf
  EndProcedure
  
  Procedure ParserData(ClientID, *Buf, Size)
    Protected String.s
    Protected Pos, Pos1, Pos2
    
    String=PeekS(*Buf, Size, #PB_Ascii) ; Информация, переданная браузером
    ReplaceString(String, Chr(13),Chr(10), #PB_String_InPlace)
    String=ReplaceString(String, Chr(10)+Chr(10), Chr(10))
    String=StringField(String, 1, Chr(10)) ; Выделяем первую строку
    Pos=FindString(LCase(String),"get",1)  ; Это GET запрос?
    If Pos>0
      Pos1=FindString(String,"/", Pos)     ; Определяем начало
      If Pos1>0
        Pos2=FindString(String," ",Pos1)   ; и конец области переменных.
        If Pos2>0
          String=Mid(String, Pos1+1, Pos2-Pos1-1)
          String=LCase(Trim(String))
          
          If String="" Or String="index.html" 
            If *g_HTML_Data
              Send_Data(ClientID, "200 OK", *g_HTML_Data, MemorySize(*g_HTML_Data))
            Else
              Send_Data(ClientID, "500 Internal Server Error", 0, 0)
            EndIf
          ElseIf String="favicon.ico"
            Send_Data(ClientID, "200 OK", ?favicon, ?end_favicon-?favicon, "image/x-icon")
          Else
            Send_Data(ClientID, "404 Not Found", 0, 0)
          EndIf
          
          
        EndIf
      EndIf
    EndIf
    
    DataSection
      favicon:
      IncludeBinary "favicon.ico"
      end_favicon:
    EndDataSection
  EndProcedure
  
  
  Procedure StartServer(Port.u, IPv6=#False)
    
    StopServer()
    
    If IPv6 = #False
      g_ServerID = CreateNetworkServer(#ServerID, Port)
    Else
      g_ServerID = CreateNetworkServer(#ServerID, Port, #PB_Network_TCP|#PB_Network_IPv6)
    EndIf
    
    ProcedureReturn Bool(g_ServerID)
  EndProcedure
  
  Procedure StopServer()
    If g_ServerID
      CloseNetworkServer(#ServerID)
      g_ServerID = 0
    EndIf
  EndProcedure
  
  Procedure ServerEvent()
    Protected Event = NetworkServerEvent(#ServerID)
    Protected ClientID, *mem, Size
    
    If Event=#PB_NetworkEvent_Data
      ClientID = EventClient()
      *mem = AllocateMemory(4096)
      If *mem
        Size=ReceiveNetworkData(ClientID, *mem, 4096)
        If Size>0
          ParserData(ClientID, *mem, Size)
        EndIf
        FreeMemory(*mem)
      EndIf
    EndIf
  EndProcedure
  
  Procedure UpdateInfo(List Sensors.SensorInfo())
    Protected s.s, x.s=""
    
    If *g_HTML_Data
      FreeMemory(*g_HTML_Data)
      *g_HTML_Data=0
    EndIf
    
    s = PeekS(?HTML, -1, #PB_UTF8)
    
    ForEach Sensors()
      x+~"<tr><td style=\"text-align: center;\">"+Sensors()\Name+"</td>"+
        ~"<td style=\"text-align: center;\">"+Sensors()\Temp+"</td></tr>"
    Next
    
    s=ReplaceString(s, "<!--TABLE-->", x, #PB_String_CaseSensitive, 1, 1)
    
    *g_HTML_Data = UTF8(s)
    
    DataSection
      HTML:
      IncludeBinary "WebPage.html"
      Data.l 0
    EndDataSection
  EndProcedure
  
EndModule
; IDE Options = PureBasic 5.70 LTS beta 1 (Windows - x86)
; CursorPosition = 78
; FirstLine = 53
; Folding = --
; EnableXP
; DPIAware